/***************************************************************************
 *                                                                         *
 *   cad/cadlayerlist.cpp                                                  *
 *   (C) 2004 by Christoph Permes                                          *
 *   ConstructCAD Project - Version 0.1                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "cadlayerlist.h"

#include <qobject.h>

using namespace cad;

/***************************************************************************
 * Construction/Destruction                                                *
 ***************************************************************************/

/**
 * Constructor
 * Create an CADLayerList with the layer "0"
 */
CADLayerList::CADLayerList()
{
	add(CADLayer("0"));
	setActive(0);
}

/** Destructor */
CADLayerList::~CADLayerList()
{

}

/***************************************************************************
 * Get/Set Methods                                                         *
 ***************************************************************************/

/** @return the layer at given index */
CADLayer CADLayerList::getLayer(int index) const throw (Exception)
{
	if ((index < 0) || (index >= getCount()))
		throw Exception(QObject::tr("Layer doesn\'t exist."));
	return layerList.at(index);
}

/** @return the layer with the given name */
CADLayer CADLayerList::getLayer(QString name) const throw (Exception)
{
	int index = find(name);
	if (index == -1)
		throw Exception(QObject::tr("Layer doesn\'t exist."));
	return layerList.at(index);
}

/** set the active layer to layerList[index] */
void CADLayerList::setActive(int index) throw (Exception)
{
	if ((index < 0) || (index >= getCount()))
		throw Exception(QObject::tr("Layer doesn\'t exist."));
	active = index;
}

/** set the layer with the given name as active layer */
void CADLayerList::setActive(QString name) throw (Exception)
{
	int index = find(name);
	if (index == -1)
		throw Exception(QObject::tr("Layer doesn\'t exist."));
	active = index;
}

/***************************************************************************
 * Public Methods                                                          *
 ***************************************************************************/

/** add layer to list if no layer with this name exists */
void CADLayerList::add(CADLayer layer) throw (Exception)
{
	if (find(layer.getName()) != -1)
		throw Exception(QObject::tr("Layer %1 already exists.").arg("\""+layer.getName()+"\""));
	layerList.push_back(layer);
}

/** remove layer at given index */
void CADLayerList::remove(int index) throw (Exception)
{
	if ((index < 0) || (index >= getCount()))
		throw Exception(QObject::tr("Layer doesn\'t exist."));
	if (index == active)
		throw Exception(QObject::tr("Can't delete active Layer."));
	if (layerList.at(index).getName() == "0")
		throw Exception(QObject::tr("Layer 0 can\'t be deleted."));
	layerList.erase(layerList.begin()+index);
}

/** remove layer with the given name */
void CADLayerList::remove(QString name) throw (Exception)
{
	int index = find(name);
	if (index == -1)
		throw Exception(QObject::tr("Layer %1 doesn\'t exist.").arg("\""+name+"\""));
	remove(index);
}

/** update layer with the given index */
void CADLayerList::update(int index, CADLayer layer) throw (Exception)
{
	if ((index < 0) || (index >= getCount()))
		throw Exception(QObject::tr("Layer doesn\'t exist."));
	int found = find(layer.getName());
	if ((found != -1) && (found != index))
		throw Exception(QObject::tr("Layer %1 already exists.").arg("\""+layer.getName()+"\""));
	if ((layerList.at(index).getName() == "0") && (layer.getName() != "0"))
		throw Exception(QObject::tr("Layer 0 can\'t be renamed"));
	layerList.at(index) = layer;
}

/** update layer with the given name */
void CADLayerList::update(QString name, CADLayer layer) throw (Exception)
{
	int index = find(name);
	if (index == -1)
		throw Exception(QObject::tr("Layer %1 doesn\'t exist.").arg("\""+name+"\""));
	update(index, layer);
}

/**
 * @return index of layer with the given name, if layer doesn't
 * exists return -1
 */
int CADLayerList::find(QString name) const
{
	for (int i = 0; i<getCount(); i++)
	{
		if (layerList.at(i).getName() == name)
			return i;
	}
	return -1;
}

// EOF
