/***************************************************************************
 *                                                                         *
 *   gui/dialogs/colordialog.cpp                                           *
 *   (C) 2004 by Christoph Permes                                          *
 *   ConstructCAD Project - Version 0.1                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
 
#include "colordialog.h"

#include "../../cad/cadlayer.h"
#include "../../io/drawingmanager.h"

#include <qmessagebox.h>

using gui::dialogs::ColorDialog;
using namespace cad;
using util::Exception;
using io::DrawingManager;

/* TRANSLATOR gui::dialogs::ColorDialog */

/***************************************************************************
 * Construction/Destruction                                                *
 ***************************************************************************/

/**
 * Constructor
 * Create a ColorDialog with the given color selected
 * @param layer: if layer is QString::null BYLAYER and BYBLOCK 
 * will be disabled
 */
ColorDialog::ColorDialog(CADColor color, QString layer, QWidget* parent, const char* name) : QDialog(parent, name)
{
	this->setCaption(tr("Choose Color"));
	this->setFixedSize(400, 280);
	this->color = color;
	this->layer = layer;
	
	standardGroup = new QGroupBox(tr("Standard Colors"), this);
	standardGroup->setGeometry(10, 10, 200, 50);
	standardColors = new ColorPalette(standardGroup);
	standardColors->setGeometry(10, 20, 180, 20);
	connect(standardColors, SIGNAL(selected(int)), this,
		SLOT(sltColorChanged(int)));
	
	grayGroup = new QGroupBox(tr("Shades of Gray"), this);
	grayGroup->setGeometry(10, 60, 140, 50);
	grayColors = new GrayPalette(grayGroup);
	grayColors->setGeometry(10, 20, 120, 20);
	connect(grayColors, SIGNAL(selected(int)), this,
		SLOT(sltColorChanged(int)));
	
	assignmentGroup = new QGroupBox(tr("Color Assignment"), this);
	assignmentGroup->setGeometry(160, 60, 230, 50);
	byLayer = new QPushButton(assignmentGroup);
	byLayer->setText("BY&LAYER");
	byLayer->setGeometry(10, 20, 100, 25);
	byLayer->setEnabled(layer != QString::null);
	connect(byLayer, SIGNAL(clicked()), this, SLOT(sltByLayer()));
	byBlock = new QPushButton(assignmentGroup);
	byBlock->setText("BY&BLOCK");
	byBlock->setGeometry(120, 20, 100, 25);
	byBlock->setEnabled(layer != QString::null);
	connect(byBlock, SIGNAL(clicked()), this, SLOT(sltByBlock()));
	
	completeGroup = new QGroupBox(tr("Complete Color Palette"), this);
	completeGroup->setGeometry(10, 120, 380, 110);
	completeColors = new CompletePalette(completeGroup);
	completeColors->setGeometry(10, 20, 360, 80);
	connect(completeColors, SIGNAL(selected(int)), this,
		SLOT(sltColorChanged(int)));
		
	colorLabel = new QLabel(this);
	colorLabel->setText(tr("Color:"));
	colorLabel->setGeometry(10, 240, 35, 20);
	
	colorName = new QLineEdit(this);
	colorName->setGeometry(50, 240, 100, 20);
	connect(colorName, SIGNAL(returnPressed()), this,
		SLOT(sltColorNameReturn()));
	
	colorDisplay = new QWidget(this);
	colorDisplay->setGeometry(160, 240, 20, 20);
	
	ok = new QPushButton(this);
	ok->setGeometry(200, 240, 90, 30);
	ok->setText(tr("&OK"));
	ok->setDefault(true);
	connect(ok, SIGNAL(clicked()), this, SLOT(sltOKClick()));
	
	cancel = new QPushButton(this);
	cancel->setGeometry(300, 240, 90, 30);
	cancel->setText(tr("&Cancel"));
	connect(cancel, SIGNAL(clicked()), this, SLOT(sltCancelClick()));
	
	updateColorDisplay();
}

/** Destructor */
ColorDialog::~ColorDialog()
{
	
}

/***************************************************************************
 * Get/Set Methods                                                         *
 ***************************************************************************/

CADColor ColorDialog::getColor()
{
	return color;
}

/***************************************************************************
 * Public Methods                                                          *
 ***************************************************************************/

/** Converts a color name to the color value */
CADColor ColorDialog::nameToValue(QString name) throw (Exception)
{
	if (name == "BYBLOCK") return CADColor::BYBLOCK;
	if (name == "red") return CADColor::RED;
	if (name == "yellow") return CADColor::YELLOW;
	if (name == "green") return CADColor::GREEN;
	if (name == "cyan") return CADColor::CYAN;
	if (name == "blue") return CADColor::BLUE;
	if (name == "magenta") return CADColor::MAGENTA;
	if (name == "white") return CADColor::WHITE;
	if (name == "BYLAYER") return CADColor::BYLAYER;
	
	bool valid;
	int value = name.toInt(&valid);
	if ((!valid) || (value < 0) || (value > 256))
		throw Exception(tr("Invalid color name."));
	return CADColor(value);
}

/** Converts a color value to the color name */
QString ColorDialog::valueToName(CADColor color)
{
	switch (color.getValue())
	{
	case 0: return "BYBLOCK";
	case 1: return "red";
	case 2: return "yellow";
	case 3: return "green";
	case 4: return "cyan";
	case 5: return "blue";
	case 6: return "magenta";
	case 7: return "white";
	case 256: return "BYLAYER";
	default:
		if ((color.getValue() > 7) && (color.getValue() < 256))
			return QString::number(color.getValue());
	}
	return QString::null;
}

/***************************************************************************
 * Private Methods                                                         *
 ***************************************************************************/

/** update the color display (name, color) */
void ColorDialog::updateColorDisplay()
{
	colorName->setText(valueToName(color.getValue()));
	CADColor c = color;
	if (c == CADColor::BYLAYER)
	{
		try
		{
			c = DrawingManager::getInstance()->getDrawing()->getLayerList()
			.getLayer(layer).getPen().getColor();
		}
		catch (...)
		{
			c.setValue(7);
			QMessageBox::information(this, tr("Error"), tr("Couldn\'t get color from layer."));
		}
	}
	if (c == CADColor::BYBLOCK)
		c.setValue(7);
	colorDisplay->setBackgroundColor(QColor(c.getRed(), c.getGreen(), 
		c.getBlue()));
}

/***************************************************************************
 * Private Slots                                                           *
 ***************************************************************************/

void ColorDialog::sltByLayer()
{
	color = CADColor::BYLAYER;
	updateColorDisplay();
}

void ColorDialog::sltByBlock()
{
	color = CADColor::BYBLOCK;
	updateColorDisplay();
}

void ColorDialog::sltColorChanged(int color)
{
	this->color.setValue(color);
	updateColorDisplay();
}

void ColorDialog::sltColorNameReturn()
{
	CADColor color;
	try
	{
		color = nameToValue(colorName->text());
		if ((layer == QString::null) && ((color == CADColor::BYLAYER) || (color == CADColor::BYBLOCK)))
		{
			QMessageBox::warning(this, tr("Error"), tr("Color can\'t be %1.").arg(colorName->text()));
			return;
		}
		this->color = color;
		updateColorDisplay();
	}
	catch (Exception e)
	{
		QMessageBox::warning(this, tr("Error"), e.getError());
	}
}

void ColorDialog::sltOKClick()
{
	accept();
}

void ColorDialog::sltCancelClick()
{
	reject();
}

// EOF
