/***************************************************************************
 *                                                                         *
 *   gui/dialogs/propertiesdialog.cpp                                      *
 *   (C) 2004 by Christoph Permes                                          *
 *   ConstructCAD Project - Version 0.1                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "propertiesdialog.h"
#include "../edit/arcedit.h"
#include "../edit/circleedit.h"
#include "../edit/ellipseedit.h"
#include "../edit/lineedit.h"
#include "../edit/pointedit.h"
#include "../edit/rayedit.h"

#include "../../cad/caddrawing.h"
#include "../../io/drawingmanager.h"

#include <qmessagebox.h>

using gui::dialogs::PropertiesDialog;
using namespace gui::edit;
using namespace cad;
using io::DrawingManager;

/* TRANSLATOR gui::dialogs::PropertiesDialog */

/***************************************************************************
 * Construction/Destruction                                                *
 ***************************************************************************/

/**
 * Constructor
 * Create a PropertiesDialog
 * @param entitiy: if entity is NULL the dialog would edit the 
 * active pen and layer settings, else the given entity is edited.
 */
PropertiesDialog::PropertiesDialog(CADEntity* entity, QWidget* parent,
	const char *name) : QDialog(parent, name)
{
	if (entity == 0)
	{
		entityEdit = 0;
		this->setCaption(tr("Object Properties"));
		CADDrawing* drawing = DrawingManager::getInstance()->getDrawing();
		properties = new PropertiesEdit(drawing->getActivePen(), drawing->getActiveLayer().getName(), this);
		properties->setGeometry(10, 10, properties->width(), properties->height());
		this->setFixedSize(properties->width()+20, 240);
	}
	else
	{
		properties = new PropertiesEdit(entity->getPen(), entity->getLayer(), this);
		properties->setGeometry(10, 10, properties->width(), properties->height());
		switch (entity->getEntityType())
		{
		case ID_CADARC:
			entityEdit = new ArcEdit((CADArc*)entity, this);
			this->setCaption(tr("Modify Arc"));
			break;
		case ID_CADCIRCLE:
			entityEdit = new CircleEdit((CADCircle*)entity, this);
			this->setCaption(tr("Modify Circle"));
			break;
		case ID_CADELLIPSE:
			entityEdit = new EllipseEdit((CADEllipse*)entity, this);
			this->setCaption(tr("Modify Ellipse"));
			break;
		case ID_CADLINE:
			entityEdit = new LineEdit((CADLine*)entity, this);
			this->setCaption(tr("Modify Line"));
			break;
		case ID_CADPOINT:
			entityEdit = new PointEdit((CADPoint*)entity, this);
			this->setCaption(tr("Modify Point"));
			break;
		case ID_CADRAY:
			entityEdit = new RayEdit((CADRay*)entity, this);
			this->setCaption(tr("Modify Ray"));
			break;
		case ID_CADXLINE:
			entityEdit = new RayEdit((CADRay*)entity, this);
			this->setCaption(tr("Modify Construction Line"));
			break;
		}
		
		this->setFixedSize(properties->width() + entityEdit->width() + 30, 240);
		entityEdit->setGeometry(260, 10, entityEdit->width(), entityEdit->height());
		connect(entityEdit, SIGNAL(specify()), this, SLOT(sltSpecify()));
	}
	
	ok = new QPushButton(this);
	ok->setGeometry(10, 200, 100, 30);
	ok->setText(tr("&OK"));
	ok->setDefault(true);
	connect(ok, SIGNAL(clicked()), this, SLOT(sltOK()));
	cancel = new QPushButton(this);
	cancel->setGeometry(120, 200, 100, 30);
	cancel->setText(tr("&Cancel"));
	connect(cancel, SIGNAL(clicked()), this, SLOT(sltCancel()));
	this->entity = entity;
}

/** Destructor */
PropertiesDialog::~PropertiesDialog()
{
	
}

/***************************************************************************
 * Get/Set Methods                                                         *
 ***************************************************************************/

/** @return the edited entity */
CADEntity* PropertiesDialog::getEntity()
{
	return entity;
}

/***************************************************************************
 * PropertiesDialog                                                        *
 ***************************************************************************/

void PropertiesDialog::sltOK()
{
	try
	{
		if (entity == 0)
		{
			CADDrawing* drawing = DrawingManager::getInstance()->getDrawing();
			drawing->setActivePen(properties->getPen());
			drawing->setActiveLayer(properties->getLayer());
		}
		else
		{
			entity = entityEdit->getEntity();
			entity->setPen(properties->getPen());
			entity->setLayer(properties->getLayer());
		}
		accept();
	}
	catch (Exception e)
	{
		QMessageBox::information(this, tr("Error"), tr(e.getError()));
	}
}

void PropertiesDialog::sltCancel()
{
	reject();
}

void PropertiesDialog::sltSpecify()
{
	QMessageBox::information(this, "TODO", "TODO: PropertiesDialog::sltSpecify()");
}

// EOF
