/***************************************************************************
 *                                                                         *
 *   gui/edit/propertiesedit.cpp                                           *
 *   (C) 2004 by Christoph Permes                                          *
 *   ConstructCAD Project - Version 0.1                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "propertiesedit.h"
#include "../dialogs/colordialog.h"
#include "../dialogs/linetypedialog.h"
#include "../dialogs/layerselectiondialog.h"
#include "../../cad/cadcolor.h"
#include "../../cad/cadlayer.h"
#include "../../io/drawingmanager.h"

#include <qmessagebox.h>

using gui::edit::PropertiesEdit;
using namespace gui::dialogs;
using namespace cad;
using io::DrawingManager;

/* TRANSLATOR gui::edit::PropertiesEdit */

/***************************************************************************
 * Construction/Destruction                                                *
 ***************************************************************************/

/**
 * Constructor
 * Create a PropertiesEdit which edits the given properties
 */
PropertiesEdit::PropertiesEdit(CADPen pen, QString layer, QWidget* parent, 
	const char* name) : QGroupBox(parent, name)
{
	this->pen = pen;
	this->layer = layer;
	this->setTitle(tr("Properties"));
	this->resize(240, 180);
	
	color = new QPushButton(this);
	color->setGeometry(10, 20, 100, 30);
	color->setText(tr("&Color"));
	connect(color, SIGNAL(clicked()), this, SLOT(sltColor()));
	colorDisplay = new QWidget(this);
	colorDisplay->setGeometry(120, 25, 20, 20);
	colorName = new QLabel(this);
	colorName->setGeometry(145, 20, 85, 30);
	
	layerChange = new QPushButton(this);
	layerChange->setGeometry(10, 60, 100, 30);
	layerChange->setText(tr("&Layer"));
	connect(layerChange, SIGNAL(clicked()), this, SLOT(sltLayer()));
	layerName = new QLabel(this);
	layerName->setGeometry(120, 60, 110, 30);
	
	linetype = new QPushButton(this);
	linetype->setGeometry(10, 100, 100, 30);
	linetype->setText(tr("Linet&ype"));
	connect(linetype, SIGNAL(clicked()), this, SLOT(sltLinetype()));
	linetypeName = new QLabel(this);
	linetypeName->setGeometry(120, 100, 110, 30);
	
	factor = new QLabel(this);
	factor->setGeometry(10, 140, 110, 30);
	factor->setText(tr("Linetypefactor:"));
	linetypeFactor = new QLineEdit(this);
	linetypeFactor->setGeometry(120, 140, 110, 25);
	connect(linetypeFactor, SIGNAL(lostFocus()), this, SLOT(sltLinetypeFactor()));
	
	update();
}

/** Destructor */
PropertiesEdit::~PropertiesEdit()
{
	
}

/***************************************************************************
 * Get/Set Methods                                                         *
 ***************************************************************************/

/** @return the pen */
CADPen PropertiesEdit::getPen() throw (Exception)
{
	bool ok;
	double value = linetypeFactor->text().toDouble(&ok);
	if (!ok)
		throw Exception(tr("The value for Linetypefactor is not valid."));
	if (value == 0.0)
		throw Exception(tr("Linetypefactor can\'t be 0.0."));
	pen.setLinetypeFactor(value);
	return pen;
}

/** @return the layer */
QString PropertiesEdit::getLayer() const
{
	return layer;
}

/***************************************************************************
 * Private Functions                                                       *
 ***************************************************************************/

void PropertiesEdit::update()
{
	colorName->setText(ColorDialog::valueToName(pen.getColor().getValue()));
	CADColor c = pen.getColor();
	if (c == CADColor::BYLAYER)
	{
		try
		{
			c = DrawingManager::getInstance()->getDrawing()->getLayerList().
				getLayer(layer).getPen().getColor();
		}
		catch (...)
		{
			c.setValue(7);
			QMessageBox::information(this, tr("Error"), tr("Couldn\'t get color from layer."));
		}
	}
	else if (c == CADColor::BYBLOCK) c = CADColor::WHITE;
	colorDisplay->setBackgroundColor(QColor(c.getRed(), c.getGreen(),
		c.getBlue()));
	layerName->setText(layer);
	linetypeName->setText(pen.getLinetype());
	QString text;
	linetypeFactor->setText(text.sprintf("%1.4f", pen.getLinetypeFactor()));
}

/***************************************************************************
 * Private Slots                                                           *
 ***************************************************************************/

void PropertiesEdit::sltColor()
{
	ColorDialog dlg(pen.getColor(), layer, this);
	if (dlg.exec() == QDialog::Accepted)
	{
		pen.setColor(dlg.getColor());
		update();
	}
}

void PropertiesEdit::sltLayer()
{
	LayerSelectionDialog dlg(DrawingManager::getInstance()->getDrawing()->getLayerList(), this);
	dlg.setLayer(layer);
	if (dlg.exec())
	{
		layer = dlg.getLayer();
		update();
	}
}

void PropertiesEdit::sltLinetype()
{
	LinetypeDialog dlg(DrawingManager::getInstance()->getDrawing()->getLinetypeList(), true, this);
	dlg.setLinetype(pen.getLinetype());
	if (dlg.exec())
	{
		pen.setLinetype(dlg.getLinetype());
		DrawingManager::getInstance()->getDrawing()->updateLinetypeList(dlg.getLinetypeList());
		update();
	}
}

void PropertiesEdit::sltLinetypeFactor()
{
	bool ok;
	double value = linetypeFactor->text().toDouble(&ok);
	if (ok)
	{
		pen.setLinetypeFactor(value);
		update();
	}
}

// EOF
