/***************************************************************************
 *                                                                         *
 *   gui/mainwindow.cpp                                                    *
 *   (C) 2004 by Christoph Permes                                          *
 *   ConstructCAD Project - Version 0.1                                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "mainwindow.h"

#include <qapplication.h>
#include <qmenubar.h>
#include <qstatusbar.h>
#include <qtoolbutton.h>
#include <qmessagebox.h>
#include <qcombobox.h>
#include <qpainter.h>

using namespace gui;
using cad::CADLayerList;

//#include "../io/iointerface.h"

using namespace io;

/* TRANSLATOR gui::MainWindow */

/***************************************************************************
 * Construction/Destruction                                                *
 ***************************************************************************/

MainWindow::MainWindow(QWidget* parent, char* name): QMainWindow(parent, name)
{
	this->setCaption("ConstructCAD");
	this->setIcon(QPixmap::fromMimeSource("icon16.xpm"));
	this->resize(800,600);
	
	drawingManager = DrawingManager::getInstance();

	createActions();
	createMenu();
	createStandardToolBar();
	createPropertiesToolBar();
	createDrawToolBar();
	createModifyToolBar();
	createOSnapToolBar();
	createDimToolBar();
	createZoomToolBar();
	initializeStatusBar();

	drawing = new QWidget(this);
	drawing->setBackgroundColor(Qt::black);
	this->setCentralWidget(drawing);
	
	commandWidget = new CommandWidget(new CommandHistory, this);
	commandWidget->setGeometry(0,
		height()-statusBar()->height()-commandWidget->height(), width(),
		commandWidget->height());
	commandWidget->getCommandLine()->setFocus();
	printLine("ConstructCAD Version 0.1");
	printLine("Have a lot of fun...");
	commandWidget->setIOState(IO_Ready);
	connect(commandWidget, SIGNAL(input(QString& )), this, SLOT(commandInput(QString& )));
	
	updatePropertiesDisplay();
	//drawingManager->updatePropertiesSelection();
}

MainWindow::~MainWindow()
{
	delete DrawingManager::getInstance();
}

/***************************************************************************
 * Public Methods                                                          *
 ***************************************************************************/

QString MainWindow::readInput() throw (Exception)
{
	return QString::null;
}

void MainWindow::print(QString s)
{
	commandWidget->print(s);
}

void MainWindow::printLine(QString s)
{
	commandWidget->printLine(s);
}

void MainWindow::draw(CADDrawing* drawing)
{

}

void MainWindow::updatePropertiesDisplay()
{
	CADDrawing* drawing = drawingManager->getDrawing();
	if ((layerSelection != 0) && (linetypeSelection != 0))
	{
		layerSelection->clear();
		linetypeSelection->clear();
		
		CADLayerList layerList = drawing->getLayerList();
		for (int i = 0; i<layerList.getCount(); i++)
		{
			CADLayer layer = layerList.getLayer(i);
			
			QPixmap color(10, 10);
			QPainter p(&color);
			p.setPen(Qt::black);
			p.setBrush(QColor(layer.getPen().getColor().getRed(), layer.getPen().getColor().getGreen(), layer.getPen().getColor().getBlue()));
			p.drawRect(0, 0, 10, 10);
			
			layerSelection->insertItem(color, layer.getName());
			
		}
		QString currentLayer = layerList.getLayer(layerList.getActive()).getName();
		for (int i = 0; i<layerSelection->count(); i++)
		{
			if (layerSelection->text(i) == currentLayer)
			{
				layerSelection->setCurrentItem(i);
				break;
			}
		}
		
		CADColor color = drawing->getActivePen().getColor();
		if (color == CADColor::BYLAYER)
		{
			// get color from layer
			color = drawing->getLayerList().getLayer(
			drawing->getLayerList().getActive()).getPen().getColor();
			// load "bylayer" icon
			QPixmap colorDisplay(QPixmap::fromMimeSource("bylayer.xpm"));
			QPainter p(&colorDisplay);
			// draw color from layer into the icon
			QPointArray points(3);
			points.setPoint(0, 3, 1);
			points.setPoint(1, 14, 1);
			points.setPoint(2, 14, 12);
			p.setPen(QColor(color.getRed(), color.getGreen(), color.getBlue()));
			p.setBrush(QColor(color.getRed(), color.getGreen(), color.getBlue()));
			p.drawPolygon(points);
			propertiesColorAction->setIconSet(QIconSet(colorDisplay));
		}
		else if (color == CADColor::BYBLOCK)
		{
			propertiesColorAction->setIconSet(QIconSet(QPixmap::fromMimeSource("byblock.xpm")));
		}
		else
		{
			QPixmap colorDisplay(QPixmap::fromMimeSource("color.xpm"));
			QPainter p(&colorDisplay);
			p.setPen(QColor(color.getRed(), color.getGreen(), color.getBlue()));
			p.setBrush(QColor(color.getRed(), color.getGreen(), color.getBlue()));
			p.drawRect(1, 1, 14, 14);
			propertiesColorAction->setIconSet(QIconSet(colorDisplay));
		}
		
		CADLinetypeList linetypeList = drawing->getLinetypeList();
		for (int i = 0; i<linetypeList.getCount(); i++)
		{
			linetypeSelection->insertItem(linetypeList.getLinetype(i).getName());
		}
		QString linetype = drawing->getActivePen().getLinetype();
		for (int i = 0; i<linetypeSelection->count(); i++)
		{
			if (linetypeSelection->text(i) == linetype)
			{
				linetypeSelection->setCurrentItem(i);
				break;
			}
		}
	}
}

void MainWindow::quitApplication()
{
	qApp->quit();
}

/***************************************************************************
 * Protected Methods                                                       *
 ***************************************************************************/

void MainWindow::resizeEvent(QResizeEvent* e)
{
	commandWidget->setGeometry(0,
		height()-statusBar()->height()-commandWidget->height(), width(),
		commandWidget->height());	
}

/***************************************************************************
 * Private Methods                                                         *
 ***************************************************************************/

void MainWindow::createMenu()
{
	createFileMenu();
	createEditMenu();
	createToolsMenu();
	createHelpMenu();
}

void MainWindow::createFileMenu()
{
	fileMenu = new QPopupMenu(this);
	fileNewAction->addTo(fileMenu);
	fileOpenAction->addTo(fileMenu);
	fileCloseAction->addTo(fileMenu);
	fileMenu->insertSeparator();
	fileSaveAction->addTo(fileMenu);
	fileSaveAsAction->addTo(fileMenu);
	fileMenu->insertSeparator();
	filePlotAction->addTo(fileMenu);
	fileMenu->insertSeparator();
	fileQuitAction->addTo(fileMenu);
	menuBar()->insertItem(tr("&File"), fileMenu);
}

void MainWindow::createEditMenu()
{
	editMenu = new QPopupMenu(this);
	editUndoAction->addTo(editMenu);
	editRedoAction->addTo(editMenu);
	editMenu->insertSeparator();
	editCutAction->addTo(editMenu);
	editCopyAction->addTo(editMenu);
	editPasteAction->addTo(editMenu);
	menuBar()->insertItem(tr("&Edit"), editMenu);
}

void MainWindow::createToolsMenu()
{
	toolsMenu = new QPopupMenu(this);
	createToolsDrawMenu();
	createToolsModifyMenu();
	createToolsOSnapMenu();
	createToolsDimMenu();
	createToolsZoomMenu();

	toolsMenu->insertSeparator();
	toolsToolboxesMenu = new QPopupMenu(this);
	toolsMenu->insertItem(tr("&Toolboxes"), toolsToolboxesMenu);
	toolsToolboxesDrawAction->addTo(toolsToolboxesMenu);
	toolsToolboxesModifyAction->addTo(toolsToolboxesMenu);
	toolsToolboxesOSnapAction->addTo(toolsToolboxesMenu);
	toolsToolboxesDimAction->addTo(toolsToolboxesMenu);
	toolsToolboxesZoomAction->addTo(toolsToolboxesMenu);
	toolsOptionsAction->addTo(toolsMenu);
	menuBar()->insertItem(tr("&Tools"), toolsMenu);
}

void MainWindow::createToolsDrawMenu()
{
	toolsDrawMenu = new QPopupMenu(this);
	toolsMenu->insertItem(tr("&Draw"), toolsDrawMenu);
	drawLineAction->addTo(toolsDrawMenu);
	drawXLineAction->addTo(toolsDrawMenu);
	drawRayAction->addTo(toolsDrawMenu);
	toolsDrawMenu->insertSeparator();
	drawPLineAction->addTo(toolsDrawMenu);
	drawMLineAction->addTo(toolsDrawMenu);
	toolsDrawMenu->insertSeparator();
	drawArcAction->addTo(toolsDrawMenu);
	drawArcContinueAction->addTo(toolsDrawMenu);
	toolsDrawMenu->insertSeparator();
	drawCircleAction->addTo(toolsDrawMenu);
	drawRingAction->addTo(toolsDrawMenu);
	toolsDrawMenu->insertSeparator();
	drawEllipseAction->addTo(toolsDrawMenu);
	toolsDrawMenu->insertSeparator();
	drawPolygonAction->addTo(toolsDrawMenu);
	drawRectangleAction->addTo(toolsDrawMenu);
	drawSolidAction->addTo(toolsDrawMenu);
	toolsDrawMenu->insertSeparator();
	drawPointAction->addTo(toolsDrawMenu);
	toolsDrawMenu->insertSeparator();
	drawBlockAction->addTo(toolsDrawMenu);
	drawBlockInsertAction->addTo(toolsDrawMenu);
	toolsDrawMenu->insertSeparator();
	drawHatchAction->addTo(toolsDrawMenu);
	toolsDrawMenu->insertSeparator();
	drawTextAction->addTo(toolsDrawMenu);
	drawTextSLAction->addTo(toolsDrawMenu);
}

void MainWindow::createToolsModifyMenu()
{
	toolsModifyMenu = new QPopupMenu(this);
	toolsMenu->insertItem(tr("&Modify"), toolsModifyMenu);
	modifyMoveAction->addTo(toolsModifyMenu);
	toolsModifyMenu->insertSeparator();
	modifyCopyAction->addTo(toolsModifyMenu);
	modifyOffsetAction->addTo(toolsModifyMenu);
	modifyMirrorAction->addTo(toolsModifyMenu);
	modifyArrayRectAction->addTo(toolsModifyMenu);
	modifyArrayPolarAction->addTo(toolsModifyMenu);
	toolsModifyMenu->insertSeparator();
	modifyRotateAction->addTo(toolsModifyMenu);
	toolsModifyMenu->insertSeparator();
	modifyStretchAction->addTo(toolsModifyMenu);
	modifyScaleAction->addTo(toolsModifyMenu);
	modifyLengthenAction->addTo(toolsModifyMenu);
	toolsModifyMenu->insertSeparator();
	modifyTrimAction->addTo(toolsModifyMenu);
	modifyExtendAction->addTo(toolsModifyMenu);
	toolsModifyMenu->insertSeparator();
	modifyBreakAction->addTo(toolsModifyMenu);
	toolsModifyMenu->insertSeparator();
	modifyEditTextAction->addTo(toolsModifyMenu);
	modifyHatchEditAction->addTo(toolsModifyMenu);
	toolsModifyMenu->insertSeparator();
	modifyChamferAction->addTo(toolsModifyMenu);
	modifyFilletAction->addTo(toolsModifyMenu);
	toolsModifyMenu->insertSeparator();
	modifyExplodeAction->addTo(toolsModifyMenu);
	modifyEraseAction->addTo(toolsModifyMenu);
}

void MainWindow::createToolsOSnapMenu()
{
	toolsSnapMenu = new QPopupMenu(this);
	toolsMenu->insertItem(tr("Object &Snap"), toolsSnapMenu);
	snapFromPoint->addTo(toolsSnapMenu);
	snapEndpoint->addTo(toolsSnapMenu);
	snapMidpoint->addTo(toolsSnapMenu);
	snapIntersection->addTo(toolsSnapMenu);
	snapCenter->addTo(toolsSnapMenu);
	snapQuadrant->addTo(toolsSnapMenu);
	snapPerpendicular->addTo(toolsSnapMenu);
	snapTangent->addTo(toolsSnapMenu);
	snapNode->addTo(toolsSnapMenu);
	snapBasepoint->addTo(toolsSnapMenu);
	snapNearest->addTo(toolsSnapMenu);
	snapNon->addTo(toolsSnapMenu);
	snapObject->addTo(toolsSnapMenu);
	snapCalculator->addTo(toolsSnapMenu);
}

void MainWindow::createToolsDimMenu()
{
	toolsDimensioningMenu = new QPopupMenu(this);
	toolsMenu->insertItem(tr("D&imensioning"), toolsDimensioningMenu);
	dimLinear->addTo(toolsDimensioningMenu);
	dimAligned->addTo(toolsDimensioningMenu);
	toolsDimensioningMenu->insertSeparator();
	dimRadius->addTo(toolsDimensioningMenu);
	dimDiameter->addTo(toolsDimensioningMenu);
	toolsDimensioningMenu->insertSeparator();
	dimAngular->addTo(toolsDimensioningMenu);
	toolsDimensioningMenu->insertSeparator();
	dimOrdinate->addTo(toolsDimensioningMenu);
	dimOrdinateX->addTo(toolsDimensioningMenu);
	dimOrdinateY->addTo(toolsDimensioningMenu);
	toolsDimensioningMenu->insertSeparator();
	dimBaseline->addTo(toolsDimensioningMenu);
	dimContinue->addTo(toolsDimensioningMenu);
	dimCenter->addTo(toolsDimensioningMenu);
	dimLeader->addTo(toolsDimensioningMenu);
	toolsDimensioningMenu->insertSeparator();
	dimTEdit->addTo(toolsDimensioningMenu);
	dimTERotate->addTo(toolsDimensioningMenu);
	dimTELeft->addTo(toolsDimensioningMenu);
	dimTEHome->addTo(toolsDimensioningMenu);
	dimTERight->addTo(toolsDimensioningMenu);
	toolsDimensioningMenu->insertSeparator();
	dimStyle->addTo(toolsDimensioningMenu);
	dimEdit->addTo(toolsDimensioningMenu);
}

void MainWindow::createToolsZoomMenu()
{
	toolsZoomMenu = new QPopupMenu(this);
	toolsMenu->insertItem(tr("&Zoom"), toolsZoomMenu);

	zoomInAction->addTo(toolsZoomMenu);
	zoomOutAction->addTo(toolsZoomMenu);
	zoomWindowAction->addTo(toolsZoomMenu);
	zoomAllAction->addTo(toolsZoomMenu);
	zoomPreviousAction->addTo(toolsZoomMenu);
	zoomScaleAction->addTo(toolsZoomMenu);
	zoomExtentAction->addTo(toolsZoomMenu);
	zoomCenterAction->addTo(toolsZoomMenu);
}

void MainWindow::createHelpMenu()
{
	helpMenu = new QPopupMenu(this);
	helpHelpAction->addTo(helpMenu);
	helpMenu->insertSeparator();
	helpAboutAction->addTo(helpMenu);
	menuBar()->insertItem(tr("&Help"), helpMenu);
}

void MainWindow::createStandardToolBar()
{
	standardToolBar = new QToolBar(tr("Standard"), this, this, TRUE);
	addDockWindow(standardToolBar, tr("Standard"), DockTop, TRUE);

	fileNewAction->addTo(standardToolBar);
	fileOpenAction->addTo(standardToolBar);
	fileSaveAction->addTo(standardToolBar);
	filePlotAction->addTo(standardToolBar);
	standardToolBar->addSeparator();
	editCutAction->addTo(standardToolBar);
	editCopyAction->addTo(standardToolBar);
	editPasteAction->addTo(standardToolBar);
	standardToolBar->addSeparator();
	editUndoAction->addTo(standardToolBar);
	editRedoAction->addTo(standardToolBar);
	standardToolBar->addSeparator();
	zoomInAction->addTo(standardToolBar);
	zoomOutAction->addTo(standardToolBar);
	zoomWindowAction->addTo(standardToolBar);
	zoomAllAction->addTo(standardToolBar);
	standardToolBar->addSeparator();
	helpHelpAction->addTo(standardToolBar);
}

void MainWindow::createPropertiesToolBar()
{
	propertiesToolBar = new QToolBar(tr("Properties"), this, this, TRUE);
	addDockWindow(propertiesToolBar, tr("Properties"), DockTop, TRUE);

	propertiesLayersAction->addTo(propertiesToolBar);

	layerSelection = new QComboBox(propertiesToolBar);
	layerSelection->setFixedSize(150,24);
	connect(layerSelection, SIGNAL(activated(const QString&)), this, SLOT(sltPropertiesLayerChange(const QString&)));

	propertiesColorAction->addTo(propertiesToolBar);
	propertiesLinetypeAction->addTo(propertiesToolBar);

	linetypeSelection = new QComboBox(propertiesToolBar);
	linetypeSelection->setFixedSize(150,24);
	connect(linetypeSelection, SIGNAL(activated(const QString&)), this, SLOT(sltPropertiesLinetypeChange(const QString&)));

	propertiesObjectAction->addTo(propertiesToolBar);
	propertiesPropertiesAction->addTo(propertiesToolBar);
}

void MainWindow::createDrawToolBar()
{
	drawToolBar = new QToolBar(tr("Draw"), this, this, TRUE);
	addDockWindow(drawToolBar, tr("Draw"), DockLeft, TRUE);

	drawLineAction->addTo(drawToolBar);
	drawXLineAction->addTo(drawToolBar);
	drawPLineAction->addTo(drawToolBar);
	drawMLineAction->addTo(drawToolBar);
	drawArcAction->addTo(drawToolBar);
	drawCircleAction->addTo(drawToolBar);
	drawEllipseAction->addTo(drawToolBar);
	drawPolygonAction->addTo(drawToolBar);
	drawRectangleAction->addTo(drawToolBar);
	drawPointAction->addTo(drawToolBar);
	drawBlockAction->addTo(drawToolBar);
	drawBlockInsertAction->addTo(drawToolBar);
	drawHatchAction->addTo(drawToolBar);
	drawTextAction->addTo(drawToolBar);
}

void MainWindow::createModifyToolBar()
{
	modifyToolBar = new QToolBar(tr("Modify"), this, this, TRUE);
	addDockWindow(modifyToolBar, tr("Modify"), DockLeft, TRUE);

	modifyMoveAction->addTo(modifyToolBar);
	modifyCopyAction->addTo(modifyToolBar);
	modifyOffsetAction->addTo(modifyToolBar);
	modifyMirrorAction->addTo(modifyToolBar);
	modifyArrayRectAction->addTo(modifyToolBar);
	modifyRotateAction->addTo(modifyToolBar);
	modifyStretchAction->addTo(modifyToolBar);
	modifyTrimAction->addTo(modifyToolBar);
	modifyExtendAction->addTo(modifyToolBar);
	modifyBreakAction->addTo(modifyToolBar);
	modifyChamferAction->addTo(modifyToolBar);
	modifyFilletAction->addTo(modifyToolBar);
	modifyExplodeAction->addTo(modifyToolBar);
	modifyEraseAction->addTo(modifyToolBar);
}

void MainWindow::createOSnapToolBar()
{
	osnapToolBar = new QToolBar(tr("Object Snap"), this, this, TRUE);
	addDockWindow(osnapToolBar, tr("Object Snap"), DockTornOff, TRUE);

	snapFromPoint->addTo(osnapToolBar);
	snapEndpoint->addTo(osnapToolBar);
	snapMidpoint->addTo(osnapToolBar);
	snapIntersection->addTo(osnapToolBar);
	snapCenter->addTo(osnapToolBar);
	snapQuadrant->addTo(osnapToolBar);
	snapPerpendicular->addTo(osnapToolBar);
	snapTangent->addTo(osnapToolBar);
	snapNode->addTo(osnapToolBar);
	snapBasepoint->addTo(osnapToolBar);
	snapNearest->addTo(osnapToolBar);
	snapNon->addTo(osnapToolBar);
	snapObject->addTo(osnapToolBar);
	snapCalculator->addTo(osnapToolBar);
}

void MainWindow::createDimToolBar()
{
	dimToolBar = new QToolBar(tr("Dimensioning"), this, this, TRUE);
	addDockWindow(dimToolBar, tr("Dimensioning"), DockTornOff, TRUE);
	
	dimLinear->addTo(dimToolBar);
	dimAligned->addTo(dimToolBar);
	dimRadius->addTo(dimToolBar);
	dimDiameter->addTo(dimToolBar);
	dimAngular->addTo(dimToolBar);
	dimOrdinate->addTo(dimToolBar);
	dimBaseline->addTo(dimToolBar);
	dimContinue->addTo(dimToolBar);
	dimCenter->addTo(dimToolBar);
	dimLeader->addTo(dimToolBar);
	dimTEdit->addTo(dimToolBar);
	dimStyle->addTo(dimToolBar);
}

void MainWindow::createZoomToolBar()
{
	zoomToolBar = new QToolBar(tr("Zoom"), this, this, TRUE);
	addDockWindow(zoomToolBar, tr("Zoom"), DockTornOff, TRUE);
	
	zoomInAction->addTo(zoomToolBar);
	zoomOutAction->addTo(zoomToolBar);
	zoomWindowAction->addTo(zoomToolBar);
	zoomAllAction->addTo(zoomToolBar);
	zoomPreviousAction->addTo(zoomToolBar);
	zoomScaleAction->addTo(zoomToolBar);
	zoomExtentAction->addTo(zoomToolBar);
	zoomCenterAction->addTo(zoomToolBar);
}

void MainWindow::initializeStatusBar()
{
	status[0] = new QLabel(statusBar());
	status[0]->setText(tr("Ready"));
	status[0]->setFixedWidth(100);
	statusBar()->addWidget(status[0]);
}

void MainWindow::createActions()
{
	createPropertiesActions();
	createDrawActions();
	createModifyActions();
	createSnapActions();
	createDimActions();
	createZoomActions();

	createFileActions();
	createEditActions();
	createToolsActions();
	createHelpActions();
}

void MainWindow::createFileActions()
{
	fileNewAction = new QAction(QIconSet(QPixmap::fromMimeSource("new.xpm")), tr("&New"), CTRL+Key_N, this);
	connect(fileNewAction, SIGNAL(activated()), this, SLOT(sltFileNew()));
	fileOpenAction = new QAction(QIconSet(QPixmap::fromMimeSource("open.xpm")), tr("&Open"), CTRL+Key_O, this);
	connect(fileOpenAction, SIGNAL(activated()), this, SLOT(sltFileOpen()));
	fileCloseAction = new QAction(tr("&Close"), 0, this);
	connect(fileCloseAction, SIGNAL(activated()), this, SLOT(sltFileClose()));
	fileSaveAction = new QAction(QIconSet(QPixmap::fromMimeSource("save.xpm")), tr("&Save"), CTRL+Key_S, this);
	connect(fileSaveAction, SIGNAL(activated()), this, SLOT(sltFileSave()));
	fileSaveAsAction = new QAction(tr("Save &As..."), 0, this);
	connect(fileSaveAsAction, SIGNAL(activated()), this, SLOT(sltFileSaveAs()));
	filePlotAction = new QAction(QIconSet(QPixmap::fromMimeSource("plot.xpm")), tr("&Plot"), CTRL+Key_P, this);
	connect(filePlotAction, SIGNAL(activated()), this, SLOT(sltFilePlot()));
	fileQuitAction = new QAction(tr("&Quit"), CTRL+Key_Q, this);
	connect(fileQuitAction, SIGNAL(activated()), this, SLOT(sltFileQuit()));
}

void MainWindow::createEditActions()
{
	editUndoAction = new QAction(QIconSet(QPixmap::fromMimeSource("undo.xpm")), tr("&Undo"), CTRL+Key_Z, this);
	connect(editUndoAction, SIGNAL(activated()), this, SLOT(sltEditUndo()));
	editRedoAction = new QAction(QIconSet(QPixmap::fromMimeSource("redo.xpm")), tr("&Redo"), CTRL+Key_Y, this);
	connect(editRedoAction, SIGNAL(activated()), this, SLOT(sltEditRedo()));
	editCutAction = new QAction(QIconSet(QPixmap::fromMimeSource("cut.xpm")), tr("&Cut"), CTRL+Key_X, this);
	connect(editCutAction, SIGNAL(activated()), this, SLOT(sltEditCut()));
	editCopyAction = new QAction(QIconSet(QPixmap::fromMimeSource("copy.xpm")), tr("&Copy"), CTRL+Key_C, this);
	connect(editCopyAction, SIGNAL(activated()), this, SLOT(sltEditCopy()));
	editPasteAction = new QAction(QIconSet(QPixmap::fromMimeSource("paste.xpm")), tr("&Paste"), CTRL+Key_V, this);
	connect(editPasteAction, SIGNAL(activated()), this, SLOT(sltEditPaste()));
}

void MainWindow::createToolsActions()
{
	toolsToolboxesDrawAction = new QAction(tr("&Draw"), 0, this);
	connect(toolsToolboxesDrawAction, SIGNAL(activated()), this, SLOT(sltToolboxesDraw()));
	toolsToolboxesModifyAction = new QAction(tr("&Modify"), 0, this);
	connect(toolsToolboxesModifyAction, SIGNAL(activated()), this, SLOT(sltToolboxesModify()));
	toolsToolboxesOSnapAction = new QAction(tr("Object &Snap"), 0, this);
	connect(toolsToolboxesOSnapAction, SIGNAL(activated()), this, SLOT(sltToolboxesOSnap()));
	toolsToolboxesDimAction = new QAction(tr("D&imensioning"), 0, this);
	connect(toolsToolboxesDimAction, SIGNAL(activated()), this, SLOT(sltToolboxesDim()));
	toolsToolboxesZoomAction = new QAction(tr("&Zoom"), 0, this);
	connect(toolsToolboxesZoomAction, SIGNAL(activated()), this, SLOT(sltToolboxesZoom()));
	toolsOptionsAction = new QAction(tr("&Options"), 0, this);
	connect(toolsOptionsAction, SIGNAL(activated()), this, SLOT(sltToolsOptions()));
}

void MainWindow::createHelpActions()
{
	helpHelpAction = new QAction(QIconSet(QPixmap::fromMimeSource("help.xpm")), tr("&Help"), Key_F1, this);
	connect(helpHelpAction, SIGNAL(activated()), this, SLOT(sltHelpHelp()));
	helpAboutAction = new QAction(tr("&About ConstructCAD"), 0, this);
	connect(helpAboutAction, SIGNAL(activated()), this, SLOT(sltHelpAbout()));
}

void MainWindow::createPropertiesActions()
{
	propertiesLayersAction = new QAction(QIconSet(QPixmap::fromMimeSource("layer.xpm")), tr("&Layers"), 0, this);
	connect(propertiesLayersAction, SIGNAL(activated()), this, SLOT(sltPropertiesLayers()));
	propertiesColorAction = new QAction(tr("C&olor"), 0, this);
	connect(propertiesColorAction, SIGNAL(activated()), this, SLOT(sltPropertiesColor()));
	propertiesLinetypeAction = new QAction(QIconSet(QPixmap::fromMimeSource("linetype.xpm")), tr("Linet&ype"), 0, this);
	connect(propertiesLinetypeAction, SIGNAL(activated()), this, SLOT(sltPropertiesLinetype()));
	propertiesObjectAction = new QAction(QIconSet(QPixmap::fromMimeSource("ddemodes.xpm")), tr("Ob&ject Properties"), 0, this);
	connect(propertiesObjectAction, SIGNAL(activated()), this, SLOT(sltPropertiesObject()));
    propertiesPropertiesAction = new QAction(QIconSet(QPixmap::fromMimeSource("properties.xpm")), tr("&Properties"), 0, this);
    connect(propertiesPropertiesAction, SIGNAL(activated()), this,
    	SLOT(sltPropertiesProperties()));
}

void MainWindow::createZoomActions()
{
	zoomInAction = new QAction(QIconSet(QPixmap::fromMimeSource("zoomin.xpm")), tr("Zoom &In"), 0, this);
	connect(zoomInAction, SIGNAL(activated()), this, SLOT(sltZoomIn()));
	zoomOutAction = new QAction(QIconSet(QPixmap::fromMimeSource("zoomout.xpm")), tr("Zoom &Out"), 0, this);
	connect(zoomOutAction, SIGNAL(activated()), this, SLOT(sltZoomOut()));
	zoomWindowAction = new QAction(QIconSet(QPixmap::fromMimeSource("zoomwindow.xpm")), tr("Zoom &Window"), 0, this);
	connect(zoomWindowAction, SIGNAL(activated()), this, SLOT(sltZoomWindow()));
	zoomAllAction = new QAction(QIconSet(QPixmap::fromMimeSource("zoomall.xpm")), tr("Zoom &All"), 0, this);
	connect(zoomAllAction, SIGNAL(activated()), this, SLOT(sltZoomAll()));
	zoomPreviousAction = new QAction(QIconSet(QPixmap::fromMimeSource("zoomprev.xpm")), tr("Zoom &Previous"), 0, this);
	connect(zoomPreviousAction, SIGNAL(activated()), this, SLOT(sltZoomPrevious()));
	zoomScaleAction = new QAction(QIconSet(QPixmap::fromMimeSource("zoomscale.xpm")), tr("Zoom &Scale"), 0, this);
	connect(zoomScaleAction, SIGNAL(activated()), this, SLOT(sltZoomScale()));
	zoomExtentAction = new QAction(QIconSet(QPixmap::fromMimeSource("zoomextent.xpm")), tr("Zoom &Extent"), 0, this);
	connect(zoomExtentAction, SIGNAL(activated()), this, SLOT(sltZoomExtent()));
	zoomCenterAction = new QAction(QIconSet(QPixmap::fromMimeSource("zoomcenter.xpm")), tr("Zoom &Center"), 0, this);
	connect(zoomCenterAction, SIGNAL(activated()), this, SLOT(sltZoomCenter()));
}

void MainWindow::createDrawActions()
{
	drawLineAction = new QAction(QIconSet(QPixmap::fromMimeSource("line.xpm")), tr("Line"), 0, this);
	connect(drawLineAction, SIGNAL(activated()), this, SLOT(sltDrawLine()));
	drawXLineAction = new QAction(QIconSet(QPixmap::fromMimeSource("xline.xpm")), tr("Construction Line"), 0, this);
	connect(drawXLineAction, SIGNAL(activated()), this, SLOT(sltDrawXLine()));
	drawRayAction = new QAction(QIconSet(QPixmap::fromMimeSource("ray.xpm")), tr("Ray"), 0, this);
	connect(drawRayAction, SIGNAL(activated()), this, SLOT(sltDrawRay()));
	drawPLineAction = new QAction(QIconSet(QPixmap::fromMimeSource("pline.xpm")), tr("Polyline"), 0, this);
	connect(drawPLineAction, SIGNAL(activated()), this, SLOT(sltDrawPLine()));
	drawMLineAction = new QAction(QIconSet(QPixmap::fromMimeSource("mline.xpm")), tr("Multiline"), 0, this);
	connect(drawMLineAction, SIGNAL(activated()), this, SLOT(sltDrawMLine()));
	drawArcAction = new QAction(QIconSet(QPixmap::fromMimeSource("arc.xpm")), tr("Arc"), 0, this);
	connect(drawArcAction, SIGNAL(activated()), this, SLOT(sltDrawArc()));
	drawArcContinueAction = new QAction(QIconSet(QPixmap::fromMimeSource("arccont.xpm")), tr("Continue Arc"), 0, this);
	connect(drawArcContinueAction, SIGNAL(activated()), this, SLOT(sltDrawArcContinue()));
	drawCircleAction = new QAction(QIconSet(QPixmap::fromMimeSource("circle.xpm")), tr("Circle"), 0, this);
	connect(drawCircleAction, SIGNAL(activated()), this, SLOT(sltDrawCircle()));
	drawRingAction = new QAction(QIconSet(QPixmap::fromMimeSource("ring.xpm")), tr("Ring"), 0, this);
	connect(drawRingAction, SIGNAL(activated()), this, SLOT(sltDrawRing()));
	drawEllipseAction = new QAction(QIconSet(QPixmap::fromMimeSource("ellipse.xpm")), tr("Ellipse"), 0, this);
	connect(drawEllipseAction, SIGNAL(activated()), this, 
    	SLOT(sltDrawEllipse()));
	drawPolygonAction = new QAction(QIconSet(QPixmap::fromMimeSource("polygon.xpm")), tr("Polygon"), 0, this);
	connect(drawPolygonAction, SIGNAL(activated()), this, SLOT(sltDrawPolygon()));
	drawRectangleAction = new QAction(QIconSet(QPixmap::fromMimeSource("rectangle.xpm")), tr("Rectangle"), 0, this);
	connect(drawRectangleAction, SIGNAL(activated()), this, SLOT(sltDrawRectangle()));
	drawSolidAction = new QAction(QIconSet(QPixmap::fromMimeSource("solid.xpm")), tr("Solid"), 0, this);
	connect(drawSolidAction, SIGNAL(activated()), this, SLOT(sltDrawSolid()));
	drawPointAction = new QAction(QIconSet(QPixmap::fromMimeSource("point.xpm")), tr("Point"), 0, this);
	connect(drawPointAction, SIGNAL(activated()), this, SLOT(sltDrawPoint()));
	drawBlockAction = new QAction(QIconSet(QPixmap::fromMimeSource("block.xpm")), tr("Block"), 0, this);
	connect(drawBlockAction, SIGNAL(activated()), this, SLOT(sltDrawBlock()));
	drawBlockInsertAction = new QAction(QIconSet(QPixmap::fromMimeSource("blockins.xpm")), tr("Insert Block"), 0, this);
	connect(drawBlockInsertAction, SIGNAL(activated()), this, SLOT(sltDrawBlockInsert()));
	drawHatchAction = new QAction(QIconSet(QPixmap::fromMimeSource("hatch.xpm")), tr("Hatching"), 0, this);
	connect(drawHatchAction, SIGNAL(activated()), this, SLOT(sltDrawHatch()));
	drawTextAction = new QAction(QIconSet(QPixmap::fromMimeSource("text.xpm")), tr("Text"), 0, this);
	connect(drawTextAction, SIGNAL(activated()), this, SLOT(sltDrawText()));
	drawTextSLAction = new QAction(QIconSet(QPixmap::fromMimeSource("textsl.xpm")), tr("Single-Line Text"), 0, this);
	connect(drawTextSLAction, SIGNAL(activated()), this, SLOT(sltDrawTextSL()));
}

void MainWindow::createModifyActions()
{
	modifyMoveAction = new QAction(QIconSet(QPixmap::fromMimeSource("move.xpm")), tr("Move"), 0, this);
	connect(modifyMoveAction, SIGNAL(activated()), this, SLOT(sltModifyMove()));
    modifyCopyAction = new QAction(QIconSet(QPixmap::fromMimeSource("objectcopy.xpm")), tr("Copy"), 0, this);
	connect(modifyCopyAction, SIGNAL(activated()), this, SLOT(sltModifyCopy()));
	modifyOffsetAction = new QAction(QIconSet(QPixmap::fromMimeSource("offset.xpm")), tr("Offset"), 0, this);
	connect(modifyOffsetAction, SIGNAL(activated()), this, SLOT(sltModifyOffset()));
	modifyMirrorAction = new QAction(QIconSet(QPixmap::fromMimeSource("mirror.xpm")), tr("Mirror"), 0, this);
	connect(modifyMirrorAction, SIGNAL(activated()), this, SLOT(sltModifyMirror()));
	modifyArrayRectAction = new QAction(QIconSet(QPixmap::fromMimeSource("arrayrect.xpm")), tr("Rectangular Array"), 0, this);
	connect(modifyArrayRectAction, SIGNAL(activated()), this, SLOT(sltModifyArrayRect()));
	modifyArrayPolarAction = new QAction(QIconSet(QPixmap::fromMimeSource("arraypolar.xpm")), tr("Polar Array"), 0, this);
	connect(modifyArrayPolarAction, SIGNAL(activated()), this, SLOT(sltModifyArrayPolar()));
	modifyRotateAction = new QAction(QIconSet(QPixmap::fromMimeSource("rotate.xpm")), tr("Rotate"), 0, this);
	connect(modifyRotateAction, SIGNAL(activated()), this,
		SLOT(sltModifyRotate()));
	modifyStretchAction = new QAction(QIconSet(QPixmap::fromMimeSource("stretch.xpm")), tr("Stretch"), 0, this);
	connect(modifyStretchAction, SIGNAL(activated()), this, SLOT(sltModifyStretch()));
	modifyScaleAction = new QAction(QIconSet(QPixmap::fromMimeSource("scale.xpm")), tr("Scale"), 0, this);
	connect(modifyScaleAction, SIGNAL(activated()), this, SLOT(sltModifyScale()));
	modifyLengthenAction = new QAction(QIconSet(QPixmap::fromMimeSource("lengthen.xpm")), tr("Lengthen"), 0, this);
	connect(modifyLengthenAction, SIGNAL(activated()), this, SLOT(sltModifyLengthen()));
	modifyTrimAction = new QAction(QIconSet(QPixmap::fromMimeSource("trim.xpm")), tr("Trim"), 0, this);
	connect(modifyTrimAction, SIGNAL(activated()), this, SLOT(sltModifyTrim()));
	modifyExtendAction = new QAction(QIconSet(QPixmap::fromMimeSource("extend.xpm")), tr("Extend"), 0, this);
	connect(modifyExtendAction, SIGNAL(activated()), this, SLOT(sltModifyExtend()));
	modifyBreakAction = new QAction(QIconSet(QPixmap::fromMimeSource("break.xpm")), tr("Break"), 0, this);
	connect(modifyBreakAction, SIGNAL(activated()), this, SLOT(sltModifyBreak()));
	modifyEditTextAction = new QAction(QIconSet(QPixmap::fromMimeSource("ddedit.xpm")), tr("Edit Text"), 0, this);
	connect(modifyEditTextAction, SIGNAL(activated()), this, SLOT(sltModifyEditText()));
	modifyHatchEditAction = new QAction(QIconSet(QPixmap::fromMimeSource("hatchedit.xpm")), tr("Edit Hatching"), 0, this);
	connect(modifyHatchEditAction, SIGNAL(activated()), this, SLOT(sltModifyHatchEdit()));
	modifyChamferAction = new QAction(QIconSet(QPixmap::fromMimeSource("chamfer.xpm")), tr("Chamfer"), 0, this);
	connect(modifyChamferAction, SIGNAL(activated()), this, SLOT(sltModifyChamfer()));
	modifyFilletAction = new QAction(QIconSet(QPixmap::fromMimeSource("fillet.xpm")), tr("Fillet"), 0, this);
	connect(modifyFilletAction, SIGNAL(activated()), this, SLOT(sltModifyFillet()));
	modifyExplodeAction = new QAction(QIconSet(QPixmap::fromMimeSource("explode.xpm")), tr("Explode"), 0, this);
	connect(modifyExplodeAction, SIGNAL(activated()), this, SLOT(sltModifyExplode()));
	modifyEraseAction = new QAction(QIconSet(QPixmap::fromMimeSource("erase.xpm")), tr("Erase"), 0, this);
	connect(modifyEraseAction, SIGNAL(activated()), this, SLOT(sltModifyErase()));
}

void MainWindow::createSnapActions()
{
	snapFromPoint = new QAction(QIconSet(QPixmap::fromMimeSource("from.xpm")), tr("Snap From Point"), 0, this);
	connect(snapFromPoint, SIGNAL(activated()), this, SLOT(sltSnapFromPoint()));
	snapEndpoint = new QAction(QIconSet(QPixmap::fromMimeSource("endp.xpm")), tr("Snap Endpoint"), 0, this);
	connect(snapEndpoint, SIGNAL(activated()), this, SLOT(sltSnapEndpoint()));
	snapMidpoint = new QAction(QIconSet(QPixmap::fromMimeSource("mid.xpm")), tr("Snap Midpoint"), 0, this);
	connect(snapMidpoint, SIGNAL(activated()), this, SLOT(sltSnapMidpoint()));
	snapIntersection = new QAction(QIconSet(QPixmap::fromMimeSource("int.xpm")), tr("Snap Intersection"), 0, this);
	connect(snapIntersection, SIGNAL(activated()), this, SLOT(sltSnapIntersection()));
	snapCenter = new QAction(QIconSet(QPixmap::fromMimeSource("cen.xpm")), tr("Snap Center"), 0, this);
	connect(snapCenter, SIGNAL(activated()), this, SLOT(sltSnapCenter()));
	snapQuadrant = new QAction(QIconSet(QPixmap::fromMimeSource("qua.xpm")), tr("Snap Quadrant"), 0, this);
	connect(snapQuadrant, SIGNAL(activated()), this, SLOT(sltSnapQuadrant()));
	snapPerpendicular = new QAction(QIconSet(QPixmap::fromMimeSource("per.xpm")), tr("Snap Perpendicular"), 0, this);
	connect(snapPerpendicular, SIGNAL(activated()), this, SLOT(sltSnapPerpendicular()));
	snapTangent = new QAction(QIconSet(QPixmap::fromMimeSource("tan.xpm")), tr("Snap Tangent"), 0, this);
	connect(snapTangent, SIGNAL(activated()), this, SLOT(sltSnapTangent()));
	snapNode = new QAction(QIconSet(QPixmap::fromMimeSource("nod.xpm")), tr("Snap Node"), 0, this);
	connect(snapNode, SIGNAL(activated()), this, SLOT(sltSnapNode()));
	snapBasepoint = new QAction(QIconSet(QPixmap::fromMimeSource("ins.xpm")), tr("Snap Basepoint"), 0, this);
	connect(snapBasepoint, SIGNAL(activated()), this, SLOT(sltSnapBasepoint()));
	snapNearest = new QAction(QIconSet(QPixmap::fromMimeSource("nea.xpm")), tr("Snap Nearest"), 0, this);
	connect(snapNearest, SIGNAL(activated()), this, SLOT(sltSnapNearest()));
	snapNon = new QAction(QIconSet(QPixmap::fromMimeSource("non.xpm")), tr("No Snap"), 0, this);
	connect(snapNon, SIGNAL(activated()), this, SLOT(sltSnapNon()));
	snapObject = new QAction(QIconSet(QPixmap::fromMimeSource("osnap.xpm")), tr("Object Snap"), 0, this);
	connect(snapObject, SIGNAL(activated()), this, SLOT(sltSnapObject()));
	snapCalculator = new QAction(QIconSet(QPixmap::fromMimeSource("cal.xpm")), tr("Calculator"), 0, this);
	connect(snapCalculator, SIGNAL(activated()), this, SLOT(sltSnapCalculator()));
}

void MainWindow::createDimActions()
{
	dimLinear = new QAction(QIconSet(QPixmap::fromMimeSource("dimlinear.xpm")), tr("Linear Dimensioning"), 0, this);
	connect(dimLinear, SIGNAL(activated()), this, SLOT(sltDimLinear()));
	dimAligned = new QAction(QIconSet(QPixmap::fromMimeSource("dimaligned.xpm")), tr("Aligned Dimensioning"), 0, this);
	connect(dimAligned, SIGNAL(activated()), this, SLOT(sltDimAligned()));
	dimRadius = new QAction(QIconSet(QPixmap::fromMimeSource("dimradius.xpm")), tr("Radius Dimensioning"), 0, this);
	connect(dimRadius, SIGNAL(activated()), this, SLOT(sltDimRadius()));
	dimDiameter = new QAction(QIconSet(QPixmap::fromMimeSource("dimdiameter.xpm")), tr("Diameter Dimensioning"), 0, this);
	connect(dimDiameter, SIGNAL(activated()), this, SLOT(sltDimDiameter()));
	dimAngular = new QAction(QIconSet(QPixmap::fromMimeSource("dimangular.xpm")), tr("Angular Dimensioning"), 0, this);
	connect(dimAngular, SIGNAL(activated()), this, SLOT(sltDimAngular()));
	dimOrdinate = new QAction(QIconSet(QPixmap::fromMimeSource("dimordinate.xpm")), tr("Ordinate Dimensioning"), 0, this);
	connect(dimOrdinate, SIGNAL(activated()), this, SLOT(sltDimOrdinate()));
	dimOrdinateX = new QAction(QIconSet(QPixmap::fromMimeSource("dimordinatex.xpm")), tr("X Data"), 0, this);
	connect(dimOrdinateX, SIGNAL(activated()), this, SLOT(sltDimOrdinateX()));
	dimOrdinateY = new QAction(QIconSet(QPixmap::fromMimeSource("dimordinatey.xpm")), tr("Y Data"), 0, this);
	connect(dimOrdinateY, SIGNAL(activated()), this, SLOT(sltDimOrdinateY()));
	dimBaseline = new QAction(QIconSet(QPixmap::fromMimeSource("dimbaseline.xpm")), tr("Baseline Dimensioning"), 0, this);
	connect(dimBaseline, SIGNAL(activated()), this, SLOT(sltDimBaseline()));
	dimContinue = new QAction(QIconSet(QPixmap::fromMimeSource("dimcont.xpm")), tr("Continue Dimensioning"), 0, this);
	connect(dimContinue, SIGNAL(activated()), this, SLOT(sltDimContinue()));
	dimCenter = new QAction(QIconSet(QPixmap::fromMimeSource("dimcenter.xpm")), tr("Center Dimensioning"), 0, this);
	connect(dimCenter, SIGNAL(activated()), this, SLOT(sltDimCenter()));
	dimLeader = new QAction(QIconSet(QPixmap::fromMimeSource("leader.xpm")), tr("Leader"), 0, this);
	connect(dimLeader, SIGNAL(activated()), this, SLOT(sltDimLeader()));
	dimTEdit = new QAction(QIconSet(QPixmap::fromMimeSource("dimtedit.xpm")), tr("Edit Dimensioning Text"), 0, this);
	connect(dimTEdit, SIGNAL(activated()), this, SLOT(sltDimTEdit()));
	dimTERotate = new QAction(QIconSet(QPixmap::fromMimeSource("dimterotate.xpm")), tr("Rotate"), 0, this);
	connect(dimTERotate, SIGNAL(activated()), this, SLOT(sltDimTERotate()));
	dimTELeft = new QAction(QIconSet(QPixmap::fromMimeSource("dimteleft.xpm")), tr("Left"), 0, this);
	connect(dimTELeft, SIGNAL(activated()), this, SLOT(sltDimTELeft()));
	dimTEHome = new QAction(QIconSet(QPixmap::fromMimeSource("dimtehome.xpm")), tr("Home"), 0, this);
	connect(dimTEHome, SIGNAL(activated()), this, SLOT(sltDimTEHome()));
	dimTERight = new QAction(QIconSet(QPixmap::fromMimeSource("dimteright.xpm")), tr("Right"), 0, this);
	connect(dimTERight, SIGNAL(activated()), this, SLOT(sltDimTERight()));
	dimStyle = new QAction(QIconSet(QPixmap::fromMimeSource("ddim.xpm")), tr("Dimensioning Style"), 0, this);
	connect(dimStyle, SIGNAL(activated()), this, SLOT(sltDimStyle()));
	dimEdit = new QAction(QIconSet(QPixmap::fromMimeSource("dimedit.xpm")), tr("Edit Dimensioning"), 0, this);
	connect(dimEdit, SIGNAL(activated()), this, SLOT(sltDimEdit()));
}

/***************************************************************************
 * Public Slots                                                            *
 ***************************************************************************/

void MainWindow::sltFileNew()
{
	drawingManager->fileNew();
}

void MainWindow::sltFileOpen()
{
	drawingManager->fileOpen();
}

void MainWindow::sltFileClose()
{
	drawingManager->fileClose();
}

void MainWindow::sltFileSave()
{
	drawingManager->fileSave();
}

void MainWindow::sltFileSaveAs()
{
	drawingManager->fileSaveAs();
}

void MainWindow::sltFilePlot()
{
	drawingManager->filePlot();
}

void MainWindow::sltFileQuit()
{
	quitApplication();
}

void MainWindow::sltEditUndo()
{
	drawingManager->editUndo();
}

void MainWindow::sltEditRedo()
{
	drawingManager->editRedo();
}

void MainWindow::sltEditCut()
{
	drawingManager->editCut();
}
void MainWindow::sltEditCopy()
{
	drawingManager->editCopy();
}

void MainWindow::sltEditPaste()
{
	drawingManager->editPaste();
}

void MainWindow::sltToolboxesDraw()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltToolboxesDraw()");
}

void MainWindow::sltToolboxesModify()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltToolboxesModify()");
}

void MainWindow::sltToolboxesOSnap()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltToolboxesOSnap()");
}

void MainWindow::sltToolboxesDim()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltToolboxesDim()");
}

void MainWindow::sltToolboxesZoom()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltToolboxesZoom()");
}

void MainWindow::sltToolsOptions()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltToolsOptions()");
}

void MainWindow::sltHelpHelp()
{
    drawingManager->helpHelp();
}

void MainWindow::sltHelpAbout()
{
	drawingManager->helpAbout();
}

void MainWindow::sltPropertiesLayers()
{
	drawingManager->propertiesLayers(this);
}

void MainWindow::sltPropertiesLayerChange(const QString& string)
{
	drawingManager->propertiesLayersChange(string, this);
}

void MainWindow::sltPropertiesColor()
{
	drawingManager->propertiesColor(this);
}

void MainWindow::sltPropertiesLinetype()
{
	drawingManager->propertiesLinetype(this);
}

void MainWindow::sltPropertiesLinetypeChange(const QString& string)
{
	drawingManager->propertiesLinetypeChange(string, this);
}

void MainWindow::sltPropertiesObject()
{
	drawingManager->propertiesObject(this);
}

void MainWindow::sltPropertiesProperties()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltPropertiesProperties()");
}
		
void MainWindow::sltZoomIn()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltZoomIn()");
}

void MainWindow::sltZoomOut()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltZoomOut()");
}

void MainWindow::sltZoomWindow()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltZoomWindow()");
}

void MainWindow::sltZoomAll()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltZoomAll()");
}

void MainWindow::sltZoomPrevious()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltZoomPrevious()");
}

void MainWindow::sltZoomScale()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltZoomScale()");
}

void MainWindow::sltZoomExtent()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltZoomExtent()");
}

void MainWindow::sltZoomCenter()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltZoomCenter()");
}

void MainWindow::sltDrawLine()
{
	drawingManager->drawLine();
}

void MainWindow::sltDrawXLine()
{
	drawingManager->drawXLine();
}

void MainWindow::sltDrawRay()
{
	drawingManager->drawRay();
}

void MainWindow::sltDrawPLine()
{
	drawingManager->drawPLine();
}

void MainWindow::sltDrawMLine()
{
	drawingManager->drawMLine();
}

void MainWindow::sltDrawArc()
{
	drawingManager->drawArc();
}

void MainWindow::sltDrawArcContinue()
{
	drawingManager->drawArcContinue();
}

void MainWindow::sltDrawCircle()
{
	drawingManager->drawCircle();
}

void MainWindow::sltDrawRing()
{
	drawingManager->drawRing();
}

void MainWindow::sltDrawEllipse()
{
	drawingManager->drawEllipse();
}

void MainWindow::sltDrawPolygon()
{
	drawingManager->drawPolygon();
}

void MainWindow::sltDrawRectangle()
{
	drawingManager->drawRectangle();
}

void MainWindow::sltDrawSolid()
{
	drawingManager->drawSolid();
}

void MainWindow::sltDrawPoint()
{
	drawingManager->drawPoint();
}

void MainWindow::sltDrawBlock()
{
	drawingManager->drawBlock();
}

void MainWindow::sltDrawBlockInsert()
{
	drawingManager->drawBlockInsert();
}

void MainWindow::sltDrawHatch()
{
	drawingManager->drawHatch();
}

void MainWindow::sltDrawText()
{
	drawingManager->drawText();
}

void MainWindow::sltDrawTextSL()
{
	drawingManager->drawTextSL();
}

void MainWindow::sltModifyMove()
{
	drawingManager->modifyMove();
}

void MainWindow::sltModifyCopy()
{
	drawingManager->modifyCopy();
}

void MainWindow::sltModifyOffset()
{
	drawingManager->modifyOffset();
}

void MainWindow::sltModifyMirror()
{
	drawingManager->modifyMirror();
}

void MainWindow::sltModifyArrayRect()
{
	drawingManager->modifyArrayRect();
}

void MainWindow::sltModifyArrayPolar()
{
	drawingManager->modifyArrayPolar();
}

void MainWindow::sltModifyRotate()
{
	drawingManager->modifyRotate();
}

void MainWindow::sltModifyStretch()
{
	drawingManager->modifyStretch();
}

void MainWindow::sltModifyScale()
{
	drawingManager->modifyScale();
}

void MainWindow::sltModifyLengthen()
{
	drawingManager->modifyLengthen();
}

void MainWindow::sltModifyTrim()
{
	drawingManager->modifyTrim();
}

void MainWindow::sltModifyExtend()
{
	drawingManager->modifyExtend();
}

void MainWindow::sltModifyBreak()
{
	drawingManager->modifyBreak();
}

void MainWindow::sltModifyEditText()
{
	drawingManager->modifyEditText();
}

void MainWindow::sltModifyHatchEdit()
{
	drawingManager->modifyHatchEdit();
}

void MainWindow::sltModifyChamfer()
{
	drawingManager->modifyChamfer();
}

void MainWindow::sltModifyFillet()
{
	drawingManager->modifyFillet();
}

void MainWindow::sltModifyExplode()
{
	drawingManager->modifyExplode();
}

void MainWindow::sltModifyErase()
{
	drawingManager->modifyErase();
}

void MainWindow::sltSnapFromPoint()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltSnapFromPoint()");
}

void MainWindow::sltSnapEndpoint()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltSnapEndpoint()");
}

void MainWindow::sltSnapMidpoint()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltSnapMidpoint()");
}

void MainWindow::sltSnapIntersection()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltSnapIntersection()");
}

void MainWindow::sltSnapCenter()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltSnapCenter()");
}

void MainWindow::sltSnapQuadrant()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltSnapQuadrant()");
}

void MainWindow::sltSnapPerpendicular()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltSnapPerpendicular()");
}

void MainWindow::sltSnapTangent()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltSnapTangent()");
}

void MainWindow::sltSnapNode()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltSnapNode()");
}

void MainWindow::sltSnapBasepoint()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltSnapBasepoint()");
}

void MainWindow::sltSnapNearest()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltSnapNearest()");
}

void MainWindow::sltSnapNon()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltSnapNon()");
}

void MainWindow::sltSnapObject()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltSnapObject()");
}

void MainWindow::sltSnapCalculator()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltSnapCalculator()");
}

void MainWindow::sltDimLinear()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltDimLinear()");
}

void MainWindow::sltDimAligned()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltDimAligned()");
}

void MainWindow::sltDimRadius()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltDimRadius()");
}

void MainWindow::sltDimDiameter()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltDimDiamter()");
}

void MainWindow::sltDimAngular()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltDimAngular()");
}

void MainWindow::sltDimOrdinate()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltDimOrdinate()");
}

void MainWindow::sltDimOrdinateX()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltDimOrdinateX()");
}

void MainWindow::sltDimOrdinateY()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltDimOrdinateY()");
}

void MainWindow::sltDimBaseline()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltDimBaseline()");
}

void MainWindow::sltDimContinue()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltDimContinue()");
}

void MainWindow::sltDimCenter()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltDimCenter()");
}

void MainWindow::sltDimLeader()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltDimLeader()");
}

void MainWindow::sltDimTEdit()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltDimTEdit()");
}

void MainWindow::sltDimTERotate()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltDimTERotate()");
}

void MainWindow::sltDimTELeft()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltDimTELeft()");
}

void MainWindow::sltDimTEHome()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltDimTEHome()");
}

void MainWindow::sltDimTERight()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltDimTERight()");
}

void MainWindow::sltDimStyle()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltDimStyle()");
}

void MainWindow::sltDimEdit()
{
	QMessageBox::information(this, "TODO", "TODO: MainWindow::sltDimEdit()");
}

void MainWindow::commandInput(QString& command)
{
	runCommand(command);
	commandWidget->setIOState(IO_Ready);
}

// EOF
